package com.persagy.iottransfer.client;

import com.persagy.iottransfer.communication.mina.tcp.client.TCPClientManager;
import com.persagy.iottransfer.communication.mina.udp.client.UDPClientManager;
import com.persagy.iottransfer.communication.util.IClientManager;
import com.persagy.iottransfer.config.IotProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @description:
 * @author:lgy
 * @data:2021/8/26 15:18
 */
@Service
public class IotClient {
    //项目ID和clientManager映射
    public static final Map<String, IClientManager> CLIENT_PROJECT_2_COLLECT_MANAGER_MAP = new ConcurrentHashMap<>();
    public static final Map<String, IClientManager> CLIENT_PROJECT_2_CONTROL_MANAGER_MAP = new ConcurrentHashMap<>();
    @Autowired
    IotProperties iotProperties;

    @Value("${iot.type.cloud:false}")
    private boolean cloud;

    public void init() {
        //直接连接云端（iot-collect/iot-control）的一侧
        if (cloud) {
            for (IotProperties.Client client : iotProperties.getClients()) {
                for (IotProperties.Project project : iotProperties.projects) {
                    IClientManager clientManager;
                    if ("UDP".equalsIgnoreCase(client.getProtocol())) {
                        clientManager = UDPClientManager.builder().targetIp(client.getTarget_ip()).targetPort(client.getTarget_port()).encoding("utf-8").compress(client.getCompress()).Max_size(client.getMax_size()).projectId(project.getId()).client(client).build();
                    } else {
                        clientManager = TCPClientManager.builder().ip(client.getTarget_ip()).port(client.getTarget_port()).encoding("utf-8").compress(client.getCompress()).Max_size(client.getMax_size()).projectId(project.getId()).client(client)
                                .separate(true).separateBytes(false).prefix((byte) client.getSeparate_begin().charAt(0)).suffix((byte) client.getSeparate_end().charAt(0)).build();
                    }
                    clientManager.hasRemark(100);
                    clientManager.Start();
                    if (client.getServer_is_iot_collect()) {
                        CLIENT_PROJECT_2_COLLECT_MANAGER_MAP.put(project.getId(), clientManager);
                    }
                    if (client.getServer_is_iot_control()) {
                        CLIENT_PROJECT_2_CONTROL_MANAGER_MAP.put(project.getId(), clientManager);
                    }
                }
            }
        }

    }


}
