package com.persagy.iottransfer.client;

import cn.hutool.core.thread.ThreadUtil;
import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.util.IClientManager;
import com.persagy.iottransfer.kafka.KafkaProducer;
import lombok.extern.slf4j.Slf4j;

import java.util.Map;

/**
 * @description: AlarmMessage消费线程
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2021/01/29 17:08
 * @version:V1.0
 **/
@Slf4j
public class IotClientReceiveMessageThread implements Runnable {

    KafkaProducer kafkaProducer;

    public IotClientReceiveMessageThread(KafkaProducer kafkaProducer) {
        this.kafkaProducer = kafkaProducer;
    }

    @Override
    public void run() {
        try {
            System.out.println("--init IotClientReceiveMessageThread start--");
            for (Map.Entry<String, IClientManager> entry : IotClient.CLIENT_PROJECT_2_COLLECT_MANAGER_MAP.entrySet()) {

                ThreadUtil.execAsync(() -> {
                    while (true) {
                        try {
                            Packet packet = entry.getValue().takeRece();
                            String project = entry.getKey();
                            PacketEntity packetEntity = new PacketEntity(project, packet);
                            kafkaProducer.sendFromcollect2edge(packetEntity);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    }
                }, true);

            }
            for (Map.Entry<String, IClientManager> entry : IotClient.CLIENT_PROJECT_2_CONTROL_MANAGER_MAP.entrySet()) {
                ThreadUtil.execAsync(() -> {
                    while (true) {
                        try {
                            Packet packet = entry.getValue().takeRece();
                            String project = entry.getKey();
                            PacketEntity packetEntity = new PacketEntity(project, packet);
                            kafkaProducer.sendFromcontrol2edge(packetEntity);
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                    }
                }, true);

            }
            System.out.println("--init IotClientReceiveMessageThread end--");
        } catch (Exception e) {
            log.error("消息处理失败", e);
        }
    }

}