package com.persagy.iottransfer.communication.mina.codec;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.MyEncoder;
import org.apache.mina.core.buffer.IoBuffer;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolEncoder;
import org.apache.mina.filter.codec.ProtocolEncoderOutput;

public class MinaPacketEncoder implements ProtocolEncoder {
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;

    boolean separate = false;
    boolean separateBytes = false;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;

    MyEncoder encoder;

    public MinaPacketEncoder(String ip, int port, String encoding, String aes_password, boolean compress) {
        this.ip = ip;
        this.port = port;

        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;

        this.separate = false;

        this.encoder = new MyEncoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    public MinaPacketEncoder(String ip, int port, String encoding, String aes_password, boolean compress, byte prefix,
                             byte suffix) {
        this.ip = ip;
        this.port = port;

        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.encoder = new MyEncoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    public MinaPacketEncoder(String ip, int port, String encoding, String aes_password, boolean compress,
                             byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;

        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.encoder = new MyEncoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    @Override
    public synchronized void encode(IoSession session, Object message, ProtocolEncoderOutput out) throws Exception {
        Packet request = (Packet) message;
        byte[] comm_bytes = this.encoder.encode(request);
        int comm_length = comm_bytes.length;
        IoBuffer buffer = IoBuffer.allocate(comm_length, false);
        buffer.put(comm_bytes);
        buffer.flip();
        out.write(buffer);
    }

    @Override
    public void dispose(IoSession session) throws Exception {
        // nothing to dispose
    }
}
