package com.persagy.iottransfer.communication.netty.codec;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.MyDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class NettyPacketDecoder extends ByteToMessageDecoder {
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;

    boolean separate = false;
    boolean separateBytes = false;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;

    MyDecoder decoder;
    Map<String, List<Byte>> tmpByteListMap = new HashMap<String, List<Byte>>();

    public NettyPacketDecoder(String ip, int port, String encoding, String aes_password, boolean compress) {
        this.ip = ip;
        this.port = port;

        this.encoding = encoding;
        this.compress = compress;

        this.separate = false;

        this.decoder = new MyDecoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    public NettyPacketDecoder(String ip, int port, String encoding, String aes_password, boolean compress, byte prefix,
                              byte suffix) {
        this.ip = ip;
        this.port = port;

        this.encoding = encoding;
        this.compress = compress;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.decoder = new MyDecoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    public NettyPacketDecoder(String ip, int port, String encoding, String aes_password, boolean compress,
                              byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;

        this.aes_password = aes_password;
        this.encoding = encoding;
        this.compress = compress;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.decoder = new MyDecoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, this.separate,
                this.separateBytes, this.prefix, this.suffix, this.prefixBytes, this.suffixBytes);
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        int count = in.readableBytes();
        byte[] bytes = new byte[count];
        in.readBytes(bytes);

        String remoteAddress = ctx.channel().remoteAddress().toString();
        List<Packet> packetList = this.decoder.decode(remoteAddress, bytes);
        for (int i = 0; i < packetList.size(); i++) {
            Packet packet = packetList.get(i);
            out.add(packet);
        }
    }

}
