package com.persagy.iottransfer.communication.netty.tcp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.*;
import com.persagy.iottransfer.config.IotProperties;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import lombok.extern.slf4j.Slf4j;

import java.net.InetSocketAddress;

@Slf4j
public class NettyTCPClientManager implements IClientManager {
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    private final EventLoopGroup workerGroup = new NioEventLoopGroup(1);
    String projectId;
    IotProperties.Client client;
    String ip;
    int port;
    String encoding;
    String ssl_protocol;
    String ssl_keystoretype;
    String ssl_algorithm;
    byte[] ssl_file_km;
    String ssl_password_km;
    byte[] ssl_file_tm;
    String ssl_password_tm;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean separate;
    boolean separateBytes;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    Integer IdleTime = null;
    boolean sessionClosed = false;
    IClientHandler handlerManager;
    ILogger logger;
    boolean started = false;
    String localAddress = "()";
    int remark_count = 0;
    private ChannelFuture future;
    private Bootstrap bootstrap;

    public NettyTCPClientManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, int Max_size, byte[] prefixBytes,
                                 byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size, byte prefix,
                                 byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size,
                                 byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime,
                                 int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                                 byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                                 byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPClientManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte[] prefixBytes,
                                 byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    public void setLogger(ILogger logger) {
        this.logger = logger;
    }

    @Override
    public String Name() {
        return "TCP Client " + this.ip + ":" + this.port + localAddress;
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (started) {
            return;
        }

        try {
            bootstrap = new Bootstrap();
            bootstrap.group(workerGroup).channel(NioSocketChannel.class).handler(new ClientChannelInitializer(this));
            future = bootstrap.connect(new InetSocketAddress(this.ip, this.port));

            started = true;
            log.warn(this.Name() + "\tStart");
        } catch (Exception e) {
            log.error(ExceptionUtil.Instance().GetMessage(e));
            try {
                workerGroup.shutdownGracefully();
            } catch (Exception e1) {
                e1.printStackTrace();
            }
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (!this.started) {
            return;
        }

        try {
            workerGroup.shutdownGracefully();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void AppendToSend(Packet MyPackage) {
        if (!this.started) {
            this.Start();
        }
        if (this.started) {
            future.channel().writeAndFlush(MyPackage);

            Packet packet = MyPackage;
            if (this.encoding != null) {
                log.info(this.Name() + "\tsend\t" + packet.packetString);
            } else {
                log.info(this.Name() + "\tsend\t" + HexToByte.byteToHex(packet.packetBytes));
            }
        }
    }

    public void AppendToRece(Packet MyPackage) {
        this.receList.offer(MyPackage, this.Max_size);
    }

    @Override
    public Packet PopRece() {
        Packet MyPackage = this.receList.poll();
        return MyPackage;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }

    @Override
    public String getProjectId() {
        return this.projectId;
    }

    @Override
    public IotProperties.Client getClient() {
        return this.client;
    }
}