package com.persagy.iottransfer.communication.netty.tcp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.HexToByte;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.ssl.SslHandler;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;

import java.net.SocketAddress;
import java.util.Date;

@Slf4j
@SuppressWarnings("unchecked")
public class NettyTCPServerHandler extends ChannelInboundHandlerAdapter {
    NettyTCPServerManager manager;

    public NettyTCPServerHandler(NettyTCPServerManager manager) {
        this.manager = manager;
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object message) throws Exception {
        Packet packet = (Packet) message;
        SocketAddress socketAddress = ctx.channel().remoteAddress();
        this.manager.AppendToRece(socketAddress.toString(), packet);
        if (this.manager.encoding != null) {
            log.info(this.manager.Name() + "\trece\t" + socketAddress + "\t" + packet.packetString);
        } else {
            log.info(this.manager.Name() + "\trece\t" + socketAddress
                    + "\t" + HexToByte.byteToHex(packet.packetBytes));
        }
    }

    @Override
    public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
        super.channelRegistered(ctx);

        SocketAddress socketAddress = ctx.channel().remoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, new Date());
        if (!this.manager.socketMap.containsKey(remoteAddress)) {
            this.manager.socketMap.put(remoteAddress, ctx.channel());
        }
        log.warn(this.manager.Name() + "\t" + "sessionOpened\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionOpened(ctx.channel().remoteAddress());
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);

        SocketAddress socketAddress = ctx.channel().remoteAddress();
        String remoteAddress = socketAddress.toString();
        log.warn(this.manager.Name() + "\t" + "channelActive\t" + remoteAddress);

        ctx.pipeline().get(SslHandler.class).handshakeFuture().addListener(new SSLHandshakeListener(this.manager, ctx));
    }

    @Override
    public void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
        super.channelUnregistered(ctx);

        SocketAddress socketAddress = ctx.channel().remoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, null);
        this.manager.socketMap.remove(remoteAddress);
        log.warn(this.manager.Name() + "\tsessionClosed\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionClosed(ctx.channel().remoteAddress());
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(ctx.channel().remoteAddress(), cause);
        }
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        super.userEventTriggered(ctx, evt);

        if (evt instanceof IdleStateEvent) {
            SocketAddress socketAddress = ctx.channel().remoteAddress();
            String remoteAddress = socketAddress.toString();
            log.warn(this.manager.Name() + "\tsessionIdle\t" + remoteAddress);
            ctx.channel().close();
        }
    }
}
