package com.persagy.iottransfer.communication.netty.tcp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.util.*;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class NettyTCPServerManager implements IServerManager {
    public final PacketBuffer<PacketEntity> receList = new PacketBuffer<PacketEntity>();
    String ip;
    int port;
    String encoding;
    String ssl_protocol;
    String ssl_keystoretype;
    String ssl_algorithm;
    byte[] ssl_file_km;
    String ssl_password_km;
    byte[] ssl_file_tm;
    String ssl_password_tm;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean separate;
    boolean separateBytes;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    Integer IdleTime = null;
    IServerHandler handlerManager;
    ILogger logger;
    Map<String, Date> connectDateMap = new HashMap<String, Date>();
    EventLoopGroup bossGroup = new NioEventLoopGroup(1);
    EventLoopGroup workerGroup = new NioEventLoopGroup();
    Map<String, Channel> socketMap = new HashMap<String, Channel>();
    int remark_count = 0;
    private boolean init = false;
    private ServerBootstrap bootstrap;

    public NettyTCPServerManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, int Max_size, byte[] prefixBytes,
                                 byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size, byte prefix,
                                 byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, int Max_size,
                                 byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, Integer IdleTime,
                                 int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                                 byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, boolean compress, Integer IdleTime, int Max_size,
                                 byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 Integer IdleTime, int Max_size, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;

        this.IdleTime = IdleTime;
    }

    public NettyTCPServerManager(String ip, int port, String encoding, String ssl_protocol, String ssl_keystoretype,
                                 String ssl_algorithm, byte[] ssl_file_km, String ssl_password_km, byte[] ssl_file_tm,
                                 String ssl_password_tm, boolean compress, Integer IdleTime, int Max_size, byte[] prefixBytes,
                                 byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.ssl_protocol = ssl_protocol;
        this.ssl_keystoretype = ssl_keystoretype;
        this.ssl_algorithm = ssl_algorithm;
        this.ssl_file_km = ssl_file_km;
        this.ssl_password_km = ssl_password_km;
        this.ssl_file_tm = ssl_file_tm;
        this.ssl_password_tm = ssl_password_tm;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        this.IdleTime = IdleTime;
    }

    @Override
    public void setHandlerManager(IServerHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    public void setLogger(ILogger logger) {
        this.logger = logger;
    }

    @Override
    public String Name() {
        return "TCP Server " + this.ip + ":" + this.port;
    }

    @Override
    public synchronized void Start() {
        if (this.init) {
            return;
        }

        try {
            bootstrap = new ServerBootstrap();
            bootstrap.group(bossGroup, workerGroup).channel(NioServerSocketChannel.class)
                    .childHandler(new ServerChannelInitializer(this)).childOption(ChannelOption.TCP_NODELAY, true);
            bootstrap.bind(this.ip, this.port).sync();
            init = true;
            log.warn(this.Name() + "\tStart");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            try {
                workerGroup.shutdownGracefully();
            } catch (Exception e1) {
                e1.printStackTrace();
            }
        }
    }

    @Override
    public void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new PacketEntity(), Integer.MAX_VALUE);
        }
        if (!this.init) {
            return;
        }
        try {
            workerGroup.shutdownGracefully();
            bossGroup.shutdownGracefully();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public synchronized void AllowJustSend_UDP(boolean allow) {

    }

    @Override
    public synchronized String[] GetClientAddresss() {
        return this.connectDateMap.keySet().toArray(new String[0]);
    }

    @Override
    public synchronized Date GetConnectTime(String clientAddress) {
        if (this.connectDateMap.containsKey(clientAddress)) {
            return this.connectDateMap.get(clientAddress);
        }
        return null;
    }

    public synchronized void SetConnectTime(String clientAddress, Date date) {
        this.connectDateMap.remove(clientAddress);
        if (date != null) {
            this.connectDateMap.put(clientAddress, date);
        }
    }

    @Override
    public void AppendToSend(String clientAddress, Packet MyPackage) {
        Channel session = this.socketMap.get(clientAddress);
        if (session != null) {
            session.writeAndFlush(MyPackage);

            Packet packet = MyPackage;
            if (this.encoding != null) {
                log.info(this.Name() + "\tsend\t" + packet.packetString);
            } else {
                log.info(this.Name() + "\tsend\t" + HexToByte.byteToHex(packet.packetBytes));
            }
        }
    }

    public synchronized boolean ContainsSocket(String clientAddress) {
        return this.socketMap.containsKey(clientAddress);
    }

    public void AppendToRece(String clientAddress, Packet MyPackage) {
        PacketEntity entity = new PacketEntity();
        entity.address = clientAddress;
        entity.content = MyPackage;
        this.receList.offer(entity, this.Max_size);
    }

    @Override
    public PacketEntity PopRece() {
        PacketEntity entity = this.receList.poll();
        return entity;
    }

    @Override
    public PacketEntity takeRece() throws InterruptedException {
        PacketEntity entity = this.receList.take();
        return entity;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void closeClient(String clientAddress) {
        if (this.socketMap.containsKey(clientAddress)) {
            Channel session = this.socketMap.get(clientAddress);
            session.close();
            this.socketMap.remove(clientAddress);
        }
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}