package com.persagy.iottransfer.communication.netty.tcp.server;

import com.persagy.iottransfer.communication.netty.codec.NettyPacketDecoder;
import com.persagy.iottransfer.communication.netty.codec.NettyPacketEncoder;
import com.persagy.iottransfer.communication.util.MySslContextFactory;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.ssl.SslHandler;
import io.netty.handler.timeout.IdleStateHandler;

import javax.net.ssl.SSLEngine;

public class ServerChannelInitializer extends ChannelInitializer<SocketChannel> {
    NettyTCPServerManager manager;

    public ServerChannelInitializer(NettyTCPServerManager manager) {
        this.manager = manager;
    }

    @Override
    protected void initChannel(SocketChannel ch) throws Exception {
        NettyPacketEncoder encoder;
        NettyPacketDecoder decoder;
        NettyTCPServerHandler handler = new NettyTCPServerHandler(manager);
        if (manager.separate) {
            if (manager.separateBytes) {
                encoder = new NettyPacketEncoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                        manager.compress, manager.prefixBytes, manager.suffixBytes);
                decoder = new NettyPacketDecoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                        manager.compress, manager.prefixBytes, manager.suffixBytes);
            } else {
                encoder = new NettyPacketEncoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                        manager.compress, manager.prefix, manager.suffix);
                decoder = new NettyPacketDecoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                        manager.compress, manager.prefix, manager.suffix);
            }
        } else {
            encoder = new NettyPacketEncoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                    manager.compress);
            decoder = new NettyPacketDecoder(manager.ip, manager.port, manager.encoding, manager.aes_password,
                    manager.compress);
        }

        if (manager.ssl_file_km != null || manager.ssl_file_tm != null) {
            SSLEngine engine = MySslContextFactory
                    .getContext(manager.ssl_protocol, manager.ssl_keystoretype, manager.ssl_algorithm,
                            manager.ssl_file_km, manager.ssl_file_tm, manager.ssl_password_km, manager.ssl_password_tm)
                    .createSSLEngine();
            engine.setUseClientMode(false);
            if (manager.ssl_file_tm != null) {
                engine.setNeedClientAuth(true);
            }
            ch.pipeline().addFirst("ssl", new SslHandler(engine));
        }
        if (manager.IdleTime != null) {
            ch.pipeline().addLast("idleStateHandler", new IdleStateHandler(0, 0, manager.IdleTime));
        }
        ch.pipeline().addLast(encoder, decoder, handler);
    }

}
