package com.persagy.iottransfer.communication.netty.udp.server;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.entity.PacketEntity;
import com.persagy.iottransfer.communication.entity.UDPSendEntity;
import com.persagy.iottransfer.communication.util.*;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.extern.slf4j.Slf4j;

import java.net.InetSocketAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class NettyUDPServerManager implements IServerManager {
    public final PacketBuffer<PacketEntity> receList = new PacketBuffer<PacketEntity>();
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    IServerHandler handlerManager;
    ILogger logger;
    Map<String, Date> connectDateMap = new HashMap<String, Date>();
    boolean allowJustSend = false;

    boolean started = false;
    EventLoopGroup group = new NioEventLoopGroup(1);
    Channel channel;
    MyEncoder encoder;
    MyDecoder decoder;
    int remark_count = 0;

    public NettyUDPServerManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;
    }

    public NettyUDPServerManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    public NettyUDPServerManager(String ip, int port, String encoding, String aes_password, boolean compress,
                                 int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    @Override
    public void setHandlerManager(IServerHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    public void setLogger(ILogger logger) {
        this.logger = logger;
    }

    @Override
    public String Name() {
        return "UDP Server " + this.ip + ":" + this.port;
    }

    @Override
    public synchronized void Start() {
        if (this.started) {
            return;
        }

        try {
            encoder = new MyEncoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, false, false,
                    (byte) '(', (byte) ')', null, null);
            decoder = new MyDecoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, false, false,
                    (byte) '(', (byte) ')', null, null);
            final NettyUDPServerHandler handler = new NettyUDPServerHandler(this);
            Bootstrap b = new Bootstrap();
            b.group(group).channel(NioDatagramChannel.class).option(ChannelOption.SO_BROADCAST, true)
                    .handler(new ChannelInitializer<NioDatagramChannel>() {
                        @Override
                        protected void initChannel(NioDatagramChannel ch) throws Exception {
                            ch.pipeline().addLast(handler);
                        }
                    });

            channel = b.bind(this.ip, this.port).channel();

            this.started = true;
            log.warn(this.Name() + "\tStart");
        } catch (Exception e) {
            log.error(ExceptionUtil.Instance().GetMessage(e));
        }
    }

    @Override
    public void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new PacketEntity(), Integer.MAX_VALUE);
        }
        if (!this.started) {
            return;
        }

        try {
            group.shutdownGracefully();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public synchronized void AllowJustSend_UDP(boolean allow) {
        this.allowJustSend = allow;
    }

    @Override
    public synchronized String[] GetClientAddresss() {
        return this.connectDateMap.keySet().toArray(new String[0]);
    }

    @Override
    public synchronized Date GetConnectTime(String clientAddress) {
        if (this.connectDateMap.containsKey(clientAddress)) {
            return this.connectDateMap.get(clientAddress);
        }
        return null;
    }

    public synchronized void SetConnectTime(String clientAddress, Date date) {
        this.connectDateMap.remove(clientAddress);
        if (date != null) {
            this.connectDateMap.put(clientAddress, date);
        }
    }

    @Override
    public void AppendToSend(String clientAddress, Packet packet) {
        try {
            byte[] encoded = this.encoder.encode(packet);
            int index_ = clientAddress.indexOf(':');
            String ip = clientAddress.substring(1, index_);
            int port = Integer.parseInt(clientAddress.substring(index_ + 1));
            InetSocketAddress remoteAddress = new InetSocketAddress(ip, port);
            channel.writeAndFlush(new DatagramPacket(Unpooled.copiedBuffer(encoded), remoteAddress));

            if (this.encoding != null) {
                log.info(this.Name() + "\tsend\t" + clientAddress + "\t" + packet.packetString);
            } else {
                log.info(this.Name() + "\tsend\t" + clientAddress + "\t" + HexToByte.byteToHex(packet.packetBytes));
            }
        } catch (Exception e) {
            log.error(ExceptionUtil.Instance().GetMessage(e));
        }
    }

    public UDPSendEntity PopSend() {
        return null;
    }

    @Override
    public PacketEntity PopRece() {
        PacketEntity entity = this.receList.poll();
        return entity;
    }

    @Override
    public PacketEntity takeRece() throws InterruptedException {
        PacketEntity entity = this.receList.take();
        return entity;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void closeClient(String clientAddress) {
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}
