package com.persagy.iottransfer.communication.nio.tcp.client;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.util.IClientHandler;
import com.persagy.iottransfer.communication.util.IClientManager;
import com.persagy.iottransfer.communication.util.ILogger;
import com.persagy.iottransfer.communication.util.PacketBuffer;
import com.persagy.iottransfer.config.IotProperties;
import lombok.Data;

@Data
public class NIOTCPClientManager implements IClientManager {
    public final PacketBuffer<Packet> sendList = new PacketBuffer<Packet>();
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    int buffer_size = 1024 * 16;
    long SelectTimeout = 1L;
    int SleepCount = 1000;
    IotProperties.Client client;
    String projectId;
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean separate;
    boolean separateBytes;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;
    boolean sessionClosed = false;
    NIOTCPClientThread thread;

    ILogger logger;
    int remark_count = 0;

    public NIOTCPClientManager(String ip, int port, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, int Max_size, byte prefix, byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, int Max_size, byte[] prefixBytes,
                               byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = false;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size, byte prefix,
                               byte suffix) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = false;
        this.prefix = prefix;
        this.suffix = suffix;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, boolean compress, int Max_size, byte[] prefixBytes,
                               byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    public NIOTCPClientManager(String ip, int port, String encoding, String aes_password, boolean compress,
                               int Max_size, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;

        this.separate = true;
        this.separateBytes = true;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
    }

    @Override
    public IotProperties.Client getClient() {
        return this.client;
    }

    @Override
    public String getProjectId() {
        return this.projectId;
    }

    @Override
    public String Name() {
        return "TCP Client " + this.ip + ":" + this.port + " NIO";
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (this.thread == null) {
            this.thread = new NIOTCPClientThread(this);
            this.thread.start();
        }
    }

    @Override
    public synchronized void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (this.thread != null) {
            this.thread.requestStop();
            this.thread = null;
        }
    }

    @Override
    public void AppendToSend(Packet MyPackage) {
        this.sendList.offer(MyPackage, this.Max_size);
    }

    public Packet PopSend() {
        Packet MyPackage = this.sendList.poll();
        return MyPackage;
    }

    public void AppendToRece(Packet MyPackage) {
        this.receList.offer(MyPackage, this.Max_size);
    }

    @Override
    public Packet PopRece() {
        Packet MyPackage = this.receList.poll();
        return MyPackage;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}
