package com.persagy.iottransfer.communication.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class CompressUtil {
    public static byte[] compress(byte[] bytes) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            GZIPOutputStream gzip = new GZIPOutputStream(out);
            gzip.write(bytes);
            gzip.close();
            return out.toByteArray();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static byte[] decompress(byte[] encrypted) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ByteArrayInputStream in = new ByteArrayInputStream(encrypted);
        try {
            GZIPInputStream gunzip = new GZIPInputStream(in);
            byte[] buffer = new byte[256];
            int n;
            while ((n = gunzip.read(buffer)) >= 0) {
                out.write(buffer, 0, n);
            }
            return out.toByteArray();
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static byte[] encode(byte[] bytes, Byte[] forbid_bytes) throws IOException {
        Byte n1 = null;
        Byte n2 = null;
        for (int i = -128; i < 128; i++) {
            byte tmp = (byte) i;
            boolean exist = false;
            for (byte tmp2 : forbid_bytes) {
                if (tmp2 == tmp) {
                    exist = true;
                    break;
                }
            }
            if (!exist) {
                if (n1 == null) {
                    n1 = tmp;
                } else if (n2 == null) {
                    n2 = tmp;
                    break;
                }
            }
        }

        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            for (byte tmp : bytes) {
                if (tmp == n1) {
                    out.write(n1);
                    out.write(n2);
                } else {
                    int index = -1;
                    for (int i = 0; i < forbid_bytes.length; i++) {
                        if (forbid_bytes[i] == tmp) {
                            index = i;
                            break;
                        }
                    }
                    if (index == -1) {
                        out.write(tmp);
                    } else {
                        for (int i = 0; i < index + 2; i++) {
                            out.write(n1);
                        }
                        out.write(n2);
                    }
                }
            }
            return out.toByteArray();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static byte[] decode(byte[] encrypted, Byte[] forbid_bytes) throws Exception {
        Byte n1 = null;
        Byte n2 = null;
        for (int i = -128; i < 128; i++) {
            byte tmp = (byte) i;
            boolean exist = false;
            for (byte tmp2 : forbid_bytes) {
                if (tmp2 == tmp) {
                    exist = true;
                    break;
                }
            }
            if (!exist) {
                if (n1 == null) {
                    n1 = tmp;
                } else if (n2 == null) {
                    n2 = tmp;
                    break;
                }
            }
        }

        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            int index = 0;
            while (index < encrypted.length) {
                byte tmp = encrypted[index];
                if (tmp == n1) {
                    int index_n2 = -1;
                    for (int i = index + 1; i < encrypted.length; i++) {
                        if (encrypted[i] == n2) {
                            index_n2 = i;
                            break;
                        }
                    }
                    if (index_n2 == -1) {
                        throw new Exception();
                    }
                    int n1_count = index_n2 - index;
                    if (n1_count == 1) {
                        out.write(n1);
                    } else {
                        out.write(forbid_bytes[n1_count - 2]);
                    }
                    index = index_n2 + 1;
                } else {
                    out.write(tmp);
                    index++;
                }
            }
            return out.toByteArray();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void main(String[] args) throws Exception {
        Byte[] forbid_bytes = {0x00, 0x01};
        String a = "1101070037;1;report;20160101000000;123456;1000;1;101;33.440&1101070037;1;report;20160101000001;123456;1001;1;101;33.441&1101070037;1;report;20160101000002;123456;1002;1;101;33.442&1101070037;1;report;20160101000003;123456;1003;1;101;33.443&1101070037;1;report;20160101000004;123456;1004;1;101;33.444&1101070037;1;report;20160101000005;123456;1005;1;101;33.445&1101070037;1;report;20160101000006;123456;1006;1;101;33.446&1101070037;1;report;20160101000007;123456;1007;1;101;33.447&1101070037;1;report;20160101000008;123456;1008;1;101;33.448&1101070037;1;report;20160101000009;123456;1009;1;101;33.449&1101070037;1;report;20160101000010;123456;1000;1;101;33.4400&1101070037;1;report;20160101000011;123456;1001;1;101;33.4411&1101070037;1;report;20160101000012;123456;1002;1;101;33.4422&1101070037;1;report;20160101000013;123456;1003;1;101;33.4433&1101070037;1;report;20160101000014;123456;1004;1;101;33.4444&1101070037;1;report;20160101000015;123456;1005;1;101;33.4455&1101070037;1;report;20160101000016;123456;1006;1;101;33.4466&1101070037;1;report;20160101000017;123456;1007;1;101;33.4477&1101070037;1;report;20160101000018;123456;1008;1;101;33.4488&1101070037;1;report;20160101000019;123456;1009;1;101;33.4499";
        for (int i = 0; i < 11; i++) {
            byte[] ab = a.getBytes();
            byte[] e = CompressUtil.compress(ab);
            byte[] ee = CompressUtil.encode(e, forbid_bytes);
            byte[] ed = CompressUtil.decode(ee, forbid_bytes);
            byte[] d = CompressUtil.decompress(ed);
            System.out.println(new String(d));
            System.out.println(ab.length + "\t" + ee.length);

            a = a.substring(1);
        }
    }
}