package com.persagy.iottransfer.communication.util;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.mina.codec.MinaCodecFactory;
import lombok.extern.slf4j.Slf4j;

import java.util.*;

@Slf4j
public class MyDecoder {
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;

    boolean separate = false;
    boolean separateBytes = false;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;

    Byte[] forbid_bytes;

    AESHelper aes;
    Map<String, MyStream> streamMap = new HashMap<String, MyStream>();
    Long last_minute = null;
    int info_minute = 0;
    int comm_minute = 0;
    int count_minute = 0;
    Long last_hour = null;
    int info_hour = 0;
    int comm_hour = 0;
    int count_hour = 0;

    public MyDecoder(String ip, int port, String encoding, String aes_password, boolean compress, boolean separate,
                     boolean separateBytes, byte prefix, byte suffix, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.separate = separate || separateBytes;
        this.separateBytes = separateBytes;
        this.prefix = prefix;
        this.suffix = suffix;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        if (this.aes_password != null) {
            try {
                this.aes = new AESHelper(null, HexToByte.hexToByte(this.aes_password), "decrypt");
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        if (this.separate) {
            List<Byte> byteList = new ArrayList<Byte>();
            if (this.separateBytes) {
                MinaCodecFactory.add_forbit_bytes(this.prefixBytes, byteList);
                MinaCodecFactory.add_forbit_bytes(this.suffixBytes, byteList);
            } else {
                MinaCodecFactory.add_forbit_byte(this.prefix, byteList);
                MinaCodecFactory.add_forbit_byte(this.suffix, byteList);
            }
            this.forbid_bytes = byteList.toArray(new Byte[0]);
        }
    }

    public synchronized List<Packet> decode(String remoteAddress, byte[] rece_bytes) throws Exception {
        List<Packet> result = new ArrayList<Packet>();

        if (this.separate) {
            if (!this.streamMap.containsKey(remoteAddress)) {
                MyStream stream;
                if (this.separateBytes) {
                    stream = new MyStream(this.prefixBytes, this.suffixBytes);
                } else {
                    stream = new MyStream(this.prefix, this.suffix);
                }
                this.streamMap.put(remoteAddress, stream);
            }
            MyStream stream = this.streamMap.get(remoteAddress);
            List<byte[]> bytesList = stream.Process(rece_bytes, rece_bytes.length);
            for (int i = 0; i < bytesList.size(); i++) {
                byte[] bytes = bytesList.get(i);
                Packet packet = this.process_packet(bytes, stream.head_count);
                result.add(packet);
            }
        } else {
            Packet packet = this.process_packet(rece_bytes, 0);
            result.add(packet);
        }

        return result;
    }

    public synchronized Packet process_packet(byte[] bytes, int head_count) throws Exception {
        Date currentTime = new Date();

        long curr_minute = currentTime.getTime() / (1000L * 60);
        if (last_minute != null && last_minute != curr_minute) {
            log.warn(this.ip + ":" + this.port + "\t" + "rece minute" + "\t" + info_minute + "\t" + comm_minute + "\t" + count_minute);
        }
        if (last_minute == null || last_minute != curr_minute) {
            last_minute = curr_minute;
            info_minute = 0;
            comm_minute = 0;
            count_minute = 0;
        }

        long curr_hour = currentTime.getTime() / (1000L * 60 * 60);
        if (last_hour != null && last_hour != curr_hour) {
            log.warn(this.ip + ":" + this.port + "\t" + "rece hour" + "\t" + info_hour + "\t" + comm_hour + "\t" + count_hour);
        }
        if (last_hour == null || last_hour != curr_hour) {
            last_hour = curr_hour;
            info_hour = 0;
            comm_hour = 0;
            count_hour = 0;
        }

        comm_minute += bytes.length + head_count;
        comm_hour += bytes.length + head_count;

        if (this.separate) {
            bytes = CompressUtil.decode(bytes, this.forbid_bytes);
        }
        if (this.compress) {
            bytes = CompressUtil.decompress(bytes);
        }
        if (this.aes_password != null) {
            try {
                bytes = this.aes.decrypt(bytes);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw e;
            }
        }

        info_minute += bytes.length;
        info_hour += bytes.length;

        count_minute++;
        count_hour++;

        Packet packet;
        if (this.encoding != null) {
            packet = new Packet(new String(bytes, this.encoding));
        } else {
            packet = new Packet(bytes);
        }
        return packet;
    }

}
