package com.persagy.iottransfer.communication.util;

import com.persagy.iottransfer.communication.entity.Packet;
import com.persagy.iottransfer.communication.mina.codec.MinaCodecFactory;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class MyEncoder {
    String ip;
    int port;
    String encoding;
    String aes_password;
    boolean compress = false;

    boolean separate = false;
    boolean separateBytes = false;
    byte prefix;
    byte suffix;
    byte[] prefixBytes;
    byte[] suffixBytes;

    Byte[] forbid_bytes;

    AESHelper aes;
    Long last_minute = null;
    long info_minute = 0;
    long comm_minute = 0;
    long count_minute = 0;
    Long last_hour = null;
    long info_hour = 0;
    long comm_hour = 0;
    long count_hour = 0;

    public MyEncoder(String ip, int port, String encoding, String aes_password, boolean compress, boolean separate,
                     boolean separateBytes, byte prefix, byte suffix, byte[] prefixBytes, byte[] suffixBytes) {
        this.ip = ip;
        this.port = port;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.separate = separate || separateBytes;
        this.separateBytes = separateBytes;
        this.prefix = prefix;
        this.suffix = suffix;
        this.prefixBytes = prefixBytes;
        this.suffixBytes = suffixBytes;

        if (this.aes_password != null) {
            try {
                this.aes = new AESHelper(null, HexToByte.hexToByte(this.aes_password), "encrypt");
            } catch (Exception e) {
                log.error(ExceptionUtil.Instance().GetMessage(e));
            }
        }
        if (this.separate) {
            List<Byte> byteList = new ArrayList<Byte>();
            if (this.separateBytes) {
                MinaCodecFactory.add_forbit_bytes(this.prefixBytes, byteList);
                MinaCodecFactory.add_forbit_bytes(this.suffixBytes, byteList);
            } else {
                MinaCodecFactory.add_forbit_byte(this.prefix, byteList);
                MinaCodecFactory.add_forbit_byte(this.suffix, byteList);
            }
            this.forbid_bytes = byteList.toArray(new Byte[0]);
        }
    }

    public synchronized byte[] encode(Packet request) throws Exception {
        Date currentTime = new Date();

        long curr_minute = currentTime.getTime() / (1000L * 60);
        if (last_minute != null && last_minute != curr_minute) {
            log.warn(this.ip + ":" + this.port + "\t" + "send minute" + "\t" + info_minute + "\t"
                    + comm_minute + "\t" + count_minute);
        }
        if (last_minute == null || last_minute != curr_minute) {
            last_minute = curr_minute;
            info_minute = 0;
            comm_minute = 0;
            count_minute = 0;
        }

        long curr_hour = currentTime.getTime() / (1000L * 60 * 60);
        if (last_hour != null && last_hour != curr_hour) {
            log.warn(this.ip + ":" + this.port + "\t" + "send hour" + "\t" + info_hour + "\t"
                    + comm_hour + "\t" + count_hour);
        }
        if (last_hour == null || last_hour != curr_hour) {
            last_hour = curr_hour;
            info_hour = 0;
            comm_hour = 0;
            count_hour = 0;
        }

        byte[] content_bytes;
        if (this.encoding != null) {
            content_bytes = request.packetString.getBytes(this.encoding);
        } else {
            content_bytes = request.packetBytes;
        }
        info_minute += content_bytes.length;
        info_hour += content_bytes.length;

        if (this.aes_password != null) {
            try {
                content_bytes = this.aes.encrypt(content_bytes);
            } catch (Exception e) {
                log.error(ExceptionUtil.Instance().GetMessage(e));
                throw e;
            }
        }
        if (this.compress) {
            content_bytes = CompressUtil.compress(content_bytes);
        }
        if (this.separate) {
            content_bytes = CompressUtil.encode(content_bytes, this.forbid_bytes);
        }

        int comm_length;
        byte[] buffer;
        if (this.separate) {
            if (this.separateBytes) {
                comm_length = this.prefixBytes.length + content_bytes.length + this.suffixBytes.length;
                buffer = new byte[comm_length];
                System.arraycopy(this.prefixBytes, 0, buffer, 0, this.prefixBytes.length);
                System.arraycopy(content_bytes, 0, buffer, this.prefixBytes.length, content_bytes.length);
                System.arraycopy(this.suffixBytes, 0, buffer, this.prefixBytes.length + content_bytes.length,
                        this.suffixBytes.length);
            } else {
                comm_length = 1 + content_bytes.length + 1;
                buffer = new byte[comm_length];
                buffer[0] = this.prefix;
                System.arraycopy(content_bytes, 0, buffer, 1, content_bytes.length);
                buffer[1 + content_bytes.length] = this.suffix;
            }
        } else {
            comm_length = content_bytes.length;
            buffer = content_bytes;
        }
        comm_minute += comm_length;
        comm_hour += comm_length;

        count_minute++;
        count_hour++;

        return buffer;
    }
}
